#include "calculator.h"

Calculator::Calculator(QWidget* pwgt) : QWidget(pwgt)
{
    m_plcd = new QLCDNumber(12);
    m_plcd->setSegmentStyle(QLCDNumber::Flat);
    m_plcd->setMinimumSize(150, 50);

    regularMode = new QRadioButton("Обычный");
    engineerMode = new QRadioButton("Инженерный");
    QVBoxLayout *modesLayout = new QVBoxLayout();
    modesLayout->addWidget(regularMode);
    modesLayout->addWidget(engineerMode);

    QString aButtons[4][5] = {{"7", "8", "9", "/", "sqrt(x)"},
                              {"4", "5", "6", "*", "1/x"},
                              {"1", "2", "3", "-", "="},
                              {"0", ".", "+"}};

    QGridLayout* ptopLayout = new QGridLayout;
    ptopLayout->addWidget(m_plcd, 0, 0, 1, 5);
    ptopLayout->addLayout(modesLayout, 1, 0, 1, 4);
    ptopLayout->addWidget(createButton("CE"), 1, 4);

    for (int i = 0; i < 2; ++i)
    {
        for (int j = 0; j < 5; ++j)
        {
            ptopLayout->addWidget(createButton(aButtons[i][j]), i + 2, j);
        }
    }

    for (int j = 0; j < 4; ++j)
    {
        ptopLayout->addWidget(createButton(aButtons[2][j]), 4, j);
    }

    QPushButton *equalBtn = createButton(aButtons[2][4]);
    equalBtn->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    ptopLayout->addWidget(equalBtn, 4, 4, 2, 1);
    ptopLayout->addWidget(createButton(aButtons[3][0]), 5, 0, 1, 2);
    ptopLayout->addWidget(createButton(aButtons[3][1]), 5, 2);
    ptopLayout->addWidget(createButton(aButtons[3][2]), 5, 3);

    setLayout(ptopLayout);
}


QPushButton* Calculator::createButton(const QString &str)
{
    QPushButton* pcmd = new QPushButton(str);
    pcmd->setMinimumSize(40, 40);
    connect(pcmd, SIGNAL(clicked()), SLOT(slotButtonClicked()));
    return pcmd;
}


void Calculator::calculate()
{
    qreal foperand2 = m_stk.pop().toFloat();
    QString strOperation = m_stk.pop();
    qreal fResult = 0;

    if (m_stk.isEmpty()) //only 1 operand
    {
        if (strOperation == "sqrt(x)")
        {
            if (foperand2 >= 0)
            {
                fResult = sqrt(foperand2);
            }
            else
            {
                m_plcd->display("ERROR");
                return;
            }
        }
        else if (strOperation == "1/x")
        {
            if (foperand2 != 0.0)
            {
                fResult = 1 / foperand2;
            }
            else
            {
                m_plcd->display("ERROR");
                return;
            }
        }
    }
    else
    {
        qreal foperand1 = m_stk.pop().toFloat();

        if(strOperation == "+")
        {
            fResult = foperand1 + foperand2;
        }
        else if (strOperation == "-")
        {
            fResult = foperand1 - foperand2;
        }
        else if (strOperation == "*")
        {
            fResult = foperand1 * foperand2;
        }
        else if (strOperation == "/")
        {
            if (foperand2 != 0.0)
            {
                fResult = foperand1 / foperand2;
            }
            else
            {
                m_plcd->display("ERROR");
                return;
            }
        }
    }

    m_plcd->display(fResult);
}


void Calculator::slotButtonClicked()
{
    QString str = ((QPushButton*)sender())->text();
    const QVector<QString> bin_ops = {"+", "-", "*", "/"};
    bool secNumInputed = false;

    if (str == "CE")
    {
        m_stk.clear();
        m_strDisplay = "";
        m_plcd->display("0");
        return;
    }

    if(str.contains(QRegularExpression("[0-9]")))
    {
        m_strDisplay += str;
        m_plcd->display(m_strDisplay.toDouble());
    }
    else if (str == ".")
    {
        m_strDisplay += str;
        m_plcd->display(m_strDisplay);
    }
    else if (std::find(bin_ops.begin(), bin_ops.end(), str) != bin_ops.end()) //binary operation
    {
        if (m_stk.count() >= 2)
        {
            m_stk.push(QString().setNum(m_plcd->value()));
            calculate();
            m_stk.clear();
            m_stk.push(QString().setNum(m_plcd->value()));
            m_stk.push(str);
        }
        else
        {
            m_stk.push(QString().setNum(m_plcd->value()));
            m_stk.push(str);
            m_strDisplay = "";
        }
    }
    else if ((str == "=") && (m_stk.count() >= 2))
    {
        m_stk.push(QString().setNum(m_plcd->value()));
        calculate();
        m_stk.push(QString().setNum(m_plcd->value()));
    }
    else //unary operation
    {
        calculate();
        m_stk.push(QString().setNum(m_plcd->value()));
    }
}
